#!/usr/bin/env fades

import argparse
import json
import os
import sys
import sqlite3
import textwrap

import tweepy  # fades.pypi

import infoauth

URL_BASE = "http://twitter.com/{username}/status/{tweet_id}"


class Twitter:
    def __init__(self):
        cfg = infoauth.load("/home/facundo/.twsearch/__tokens__.auth")

        auth = tweepy.OAuthHandler(cfg['consumer_token'], cfg['consumer_secret'])
        auth.set_access_token(cfg['access_token'], cfg['access_secret'])
        self.api = tweepy.API(auth)

    def get_timeline(self, username, max_id):
        return self.api.user_timeline(username, max_id=max_id)


class Store:
    def __init__(self, username):
        self.username = username
        self.twitter = Twitter()

        _path = "/home/facundo/.twsearch/" + username
        self.db = sqlite3.connect(_path)
        self._create()

    def _create(self):
        """Create structures in the file."""
        self.db.execute("""
            CREATE TABLE IF NOT EXISTS tweets (id INTEGER UNIQUE, tstamp TEXT, text TEXT);
        """)

    def _from_network(self, lastid, limit):
        """Get data from the network, until reaching limit, saving in DB."""
        while True:
            tweets = self.twitter.get_timeline(self.username, max_id=lastid)
            for tw in tweets:
                if tw.id == limit:
                    return
                tstamp = tw.created_at.strftime("%Y:%m:%d %H:%M:%S")
                self.db.execute("INSERT INTO tweets VALUES (?, ?, ?);", (tw.id, tstamp, tw.text))
                yield dict(tweet_id=tw.id, timestamp=tstamp, text=tw.text)
            self.db.commit()
            lastid = tw.id - 1

    def __iter__(self):
        latest_saved = self.db.execute('SELECT MAX(id) FROM tweets;').fetchall()[0][0]

        # get tweets from the network until we get to the cached ones
        yield from self._from_network(None, latest_saved)

        # yield what was cached before
        oldies = self.db.execute('SELECT * FROM tweets WHERE id <= ? ORDER BY id DESC;',
                                 (latest_saved,))
        for tw_id, tstamp, text in oldies:
            yield dict(tweet_id=tw_id, timestamp=tstamp, text=text)

        # keep getting from the network, forever
        yield from self._from_network(tw_id - 1, None)


def main(username, words):
    """Main entry point."""
    if username[0] == '@':
        username = username[1:]

    print("Searching user {!r} for words {}".format(username, words))
    store = Store(username)
    for data in store:
        print("\r... {} ...".format(data['timestamp']), end='', flush=True)
        if all(w in data['text'] for w in words):
            url = URL_BASE.format(username='facundobatista', tweet_id=data['tweet_id'])
            print("\r{}  {}".format(url, data['timestamp']))
            print(textwrap.indent(data['text'], "    "))
            print()


USAGE = """
    Usage: twitter_search.py username word1 [word2 [...]]

            it will search those words using AND
"""

if __name__ == '__main__':
    parser = argparse.ArgumentParser("Search in Twitter.")
    if len(sys.argv) < 3:
        print(USAGE)
        sys.exit()
    main(sys.argv[1], sys.argv[2:])
